/*********************************************************************
 
  █████╗ ███████╗███████╗██╗ ██████╗██╗  ██╗ █████╗  ██████╗ ███████╗
 ██╔══██╗██╔════╝██╔════╝██║██╔════╝██║  ██║██╔══██╗██╔════╝ ██╔════╝
 ███████║█████╗  █████╗  ██║██║     ███████║███████║██║  ███╗█████╗  
 ██╔══██║██╔══╝  ██╔══╝  ██║██║     ██╔══██║██╔══██║██║   ██║██╔══╝  
 ██║  ██║██║     ██║     ██║╚██████╗██║  ██║██║  ██║╚██████╔╝███████╗
 ╚═╝  ╚═╝╚═╝     ╚═╝     ╚═╝ ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝ ╚═════╝ ╚══════╝
 
 Dans cette page sont rangées toutes les fonctions permettant
 l'affichage à l'écran du rendu et autres éléments d'interface
 
 **********************************************************************/



PGraphics setUpLayer;
PGraphics clippingLayer;
PGraphics displayLayer;
PGraphics bckgrndLayer;
PGraphics prerenderingLayer;
PGraphics postProcessingLayer;
PGraphics UILayer;
PGraphics LogoLayer;
PGraphics[] hackLayer;
PImage[] pastScreen = new PImage[10];
boolean initMenu = false;
boolean portrait = false;



void layerInit() {

  /* cette fonction est lue au lancement du programme, 
   son rôle est d'initialiser chaque calque qui composera
   le rendu du programme */

  int l = pastScreen.length;
  
  clippingLayer       = createGraphics(width, height, P2D);
  setUpLayer          = createGraphics(width, height, P2D);
  displayLayer        = createGraphics(width, height, P2D);
  bckgrndLayer        = createGraphics(width, height, P2D);
  prerenderingLayer   = createGraphics(width, height, P2D);
  postProcessingLayer = createGraphics(width, height, P2D);
  UILayer             = createGraphics(width, height, P2D);
  LogoLayer           = createGraphics(width, height, P2D);
  
  font = loadFont("font/BlissPro-ExtraBold-48.vlw");

  for (int i = 0; i < l; i ++ ) pastScreen[i] = createGraphics(width, height, P2D);
  
  if (advanced) advancedInit();
  
}



void advancedInit() {

  /* cette fonction est lue au lancement du programme, son
     role est d'initialiser les claques pour les hacks avancés */
  
  hackLayer = new PGraphics[4];
  
  hackLayer[0] = createGraphics(width, height, P2D);
  hackLayer[1] = createGraphics(width, height, P2D);
  hackLayer[2] = createGraphics(width, height, P2D);
  hackLayer[3] = createGraphics(width, height, P2D);
  
}




void hackPreset(){
  
  cursorX = float(mouseX)/float(width);
  cursorY = float(mouseY)/float(height);
  
  clipping(clippingLayer);
  
  pushMatrix();
  imageMode(CENTER);
  background(colorPalette[0]);
  hack();
  popMatrix();
  
}




void advancedPreset(){
  
  pushMatrix();
  background(colorPalette[0]);
  advanced();
  popMatrix();
  
}



void display() { 

  /* cette fonction ordonne toutes le fonctions en charge de
  l'affichage dans le programe */

  boolean b = ( background[cntrlInt[3]] == null );
  boolean u = ( UI && !titleScreen );
  boolean l = ( !titleScreen );
  boolean t = ( titleScreen && !hack && !advanced ) ;
  boolean a = ( advanced ) ;
  boolean h = ( hack ) ;
  boolean p = ( !advanced ) ;
  boolean d = ( !advanced && !hack ) ;

  if ( p ) camera.read();
  
  if ( !u ) noCursor();
  else if (sliderOn) cursor(MOVE);
  else if (cursor)   cursor(HAND);
  else               cursor(ARROW);
  
  if      ( t ) titleScreen();
  else if ( b ) setupMenu();
  else if ( a ) advancedPreset();
  else if ( h ) hackPreset();
  else if ( p ) imageProcessing();
  
  displayOnScreen(b, u, l, hack);
  
  saveScreen();
  
  
}



void imageProcessing() { 

  /* cette fonction applique un à un chaque effet sur le rendu de
  la camera, ses derniers sont stockés dans des calques qui seront
  affichées par la suite */

  clipping(clippingLayer);
  displayMode(cntrlInt[0]);
  bgMode(cntrlInt[1]);
  prerendering();
  postprocessing();
  
  if ( UI ) UI();
}



void titleScreen() { 

  /* cette fonction applique un à un chaque effet sur le rendu de
  la camera, ses derniers sont stockés dans des calques qui seront
  affichées par la suite */

  displayMode(cntrlInt[0]);
  bgMode(cntrlInt[1]);
  prerendering();
  postprocessing();
  saveScreen();
  displayOnScreen(false, false, false, false);
}



void prerendering() {

  /* cette fonction crée le calque de pré-rendu sur lequel 
  s'aplliquerons par la suite les effets post processing */

  prerenderingLayer.beginDraw();
  prerenderingLayer.background(colorPalette[0]);
  prerenderingLayer.image(bckgrndLayer, 0, 0, width, height);
  prerenderingLayer.image(displayLayer, 0, 0, width, height);
  prerenderingLayer.endDraw();
}



void saveScreen() {

  /* cette fonction enregistre et stocke les dix dernières frames du
  rendu avant l'étape post processing, afin de pouvoir les utiliser
  pour l'effet motionBlur */

  PGraphics lastFrame;
  
  prerenderingLayer.loadPixels();

  lastFrame = createGraphics(width, height, P2D);
  lastFrame.beginDraw();
  lastFrame.image(prerenderingLayer, 0, 0, width, height);
  lastFrame.endDraw();

  for (int i = 0; i < pastScreen.length - 1; i++ ) pastScreen[i] = pastScreen[i+1];
  
  pastScreen[pastScreen.length-1] = lastFrame;
  
  shaderSetLastFrame();
}



void displayOnScreen(boolean b, boolean u, boolean l, boolean h) { 

  /* cette fonction affiche les différents calques à l'écran */
  
  imageMode(CORNER);

  if      ( b  ) image(setUpLayer, 0, 0, width, height);
  else if ( !h ) image(postProcessingLayer, 0, 0, width, height);
  if      ( u  ) image(UILayer, 0, 0, width, height);
  if      ( l  ) image(LogoLayer, 0, 0, width, height);
}
