/**************************************************************

   ██████╗ █████╗ ██████╗ ████████╗██╗   ██╗██████╗ ███████╗
  ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██║   ██║██╔══██╗██╔════╝
  ██║     ███████║██████╔╝   ██║   ██║   ██║██████╔╝█████╗  
  ██║     ██╔══██║██╔═══╝    ██║   ██║   ██║██╔══██╗██╔══╝  
  ╚██████╗██║  ██║██║        ██║   ╚██████╔╝██║  ██║███████╗
   ╚═════╝╚═╝  ╚═╝╚═╝        ╚═╝    ╚═════╝ ╚═╝  ╚═╝╚══════╝
   
   Dans cette page sont rangées toutes les fonctions permettant 
   la capture et le traitement de l'image perçue par la camera
   
***************************************************************/                                                          



import processing.video.*;

Capture camera;
Capture[] cameras; 
PImage[] background;



void cameraInit() {
  
  /* cette fonction initialise tout ce qui sera
  nécesaire à la prise de vue */
  
  String[] list = cameraListing();
  int l = list.length;
  boolean b = ( !advanced && !cameraBG );
  
  cameras = new Capture[l];
  background = new PImage[l];
  
  for (int i = 0; i < l ; i ++ ){
    
    cameras[i] = new Capture(this, list[i]);
  }
 
  changeCamera(0);
  
}



void changeCamera(int c) {
  
  /* cette fonction permet de changer de camera les 
  autres caméra sont mise en pause afin d'économiser
  de la bande passante */
  
  int l = cameras.length;

  for (int i = 0; i < l ; i ++ ) cameras[i].stop();
  
  camera = cameras[c];
  camera.start();
}



void capture() {
  
  /* cette fonction retourne une image correspondant
  à la derniere frame perçue par la caméra, un effet
  de le flou y est appliqué pour attenué l'éventuel
  bruit présent sur l'image qui pourrais compromettre
  la précision du détourage */

  PGraphics img;

  img = createGraphics(camera.width, camera.height, P2D);
  img.beginDraw();
  img.image(camera, 0, 0, camera.width, camera.height);
  img.filter(blur);
  img.endDraw();
  
  clippingHSB.set("background", img);
  clippingRVB.set("background", img);
  
  background[cntrlInt[3]] = img;
}



String[] cameraListing() {
  
  /* cette fonction analyse toutes les caméras reçues
  par le programme avec différents paramètres, et ne
  retiens que la première de chacune d'entre elles*/
  
  String[] list = Capture.list();
  int iL = list.length;
  String[] newList = new String[0];
  
  
  String lastName = "";
  
  // pour chaque entrée de la liste
  
  for (int i = 0; i < iL ; i ++ ) {
    
    int c = 0;
    int jL = list[i].length();
    String[] divideList = {"","",""};
    
    // divise en trois liste
    
    for (int j = 0; j < jL ; j ++ ) {
      
      char cl = list[i].charAt(j);
      
      if (cl==',') c++;
      else divideList[c] += cl;
    }
    // ne retiens que la première avec un nom différent
    
    if( !divideList[0].equals(lastName) ) newList = append(newList, divideList[0]+","+divideList[1]+","+divideList[2]);
    
    lastName = divideList[0];
  }
  
  return newList;
}
