#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif

#define PROCESSING_TEXTURE_SHADER

uniform sampler2D texture;
uniform sampler2D background;
uniform vec2 texOffset;

varying vec4 vertColor;
varying vec4 vertTexCoord;


// Converti des valeur RVB en HSB, les valeures rendues sont comprises entre 0 et 1
vec3 HSV(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

void main(void) {
  
  vec2 tc4 = vertTexCoord.st;

  vec4 cam = texture2D(texture, tc4);

  vec2 ntc4 = vec2(tc4.s, tc4.t);

  // les pixels à comparer
  vec4 cap = texture2D(background, ntc4);

  // les mêmes convertis en HSB
  vec3 c = HSV(cam.rgb);
  vec3 t = HSV(cap.rgb);
  
  // compare les valeurs
  float h=abs(c.r-t.r);
  float s=abs(c.g-t.g);
  float b=abs(c.b-t.b);

  // compare les valeurs
  vec4 r  = vec4(cam.rgb,0.0);

  if      ( h > 0.1)   r.w = 1.0;
  else if ( s > 0.5)  r.w = 1.0;
  else if ( b > 0.3)  r.w = 1.0;
  
  gl_FragColor = r;

 
}
