#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif

#define PROCESSING_TEXTURE_SHADER

uniform sampler2D texture;
uniform float opacity;
varying vec4 vertTexCoord;

uniform float time;
out vec4 fragment;

// A single iteration of Bob Jenkins' One-At-A-Time hashing algorithm.
uint hash( uint x ) {
    x += ( x << 10u );
    x ^= ( x >>  6u );
    x += ( x <<  3u );
    x ^= ( x >> 11u );
    x += ( x << 15u );
    return x;
}

// Compound versions of the hashing algorithm I whipped together.
uint hash( uvec2 v ) { return hash( v.x ^ hash(v.y)                         ); }
uint hash( uvec3 v ) { return hash( v.x ^ hash(v.y) ^ hash(v.z)             ); }
uint hash( uvec4 v ) { return hash( v.x ^ hash(v.y) ^ hash(v.z) ^ hash(v.w) ); }

// Construct a float with half-open range [0:1] using low 23 bits.
// All zeroes yields 0.0, all ones yields the next smallest representable value below 1.0.
float floatConstruct( uint m ) {
    const uint ieeeMantissa = 0x007FFFFFu; // binary32 mantissa bitmask
    const uint ieeeOne      = 0x3F800000u; // 1.0 in IEEE binary32

    m &= ieeeMantissa;                     // Keep only mantissa bits (fractional part)
    m |= ieeeOne;                          // Add fractional part to 1.0

    float  f = uintBitsToFloat( m );       // Range [1:2]
    return f - 1.0;                        // Range [0:1]
}

// Pseudo-random value in half-open range [0:1].
float random( vec3  v ) { 
  return floatConstruct(hash(floatBitsToUint(v))); 
}


void main(){
    vec2 st = vertTexCoord.st;
    vec4 cl = texture2D(texture, st);
    //vec3  inputs = vec3( round(st.x*640)/640, round(st.y*360)/360, time ); // Spatial and temporal inputs
    vec3  inputs = vec3( st, time ); // Spatial and temporal inputs
    float rand   = (random( inputs )*2-1) *  opacity;// + ( 1 - opacity/2 );              // Random per-pixel value
    vec3  luma   = vec3( cl.r+rand, cl.g+rand, cl.b+rand );                  // Expand to RGB

    fragment = vec4( luma, 1.0 ); 
}
