/*************************************************

  ███████╗██╗  ██╗ █████╗ ██████╗ ███████╗██████╗ 
  ██╔════╝██║  ██║██╔══██╗██╔══██╗██╔════╝██╔══██╗
  ███████╗███████║███████║██║  ██║█████╗  ██████╔╝
  ╚════██║██╔══██║██╔══██║██║  ██║██╔══╝  ██╔══██╗
  ███████║██║  ██║██║  ██║██████╔╝███████╗██║  ██║
  ╚══════╝╚═╝  ╚═╝╚═╝  ╚═╝╚═════╝ ╚══════╝╚═╝  ╚═╝

  Dans cette page sont rangées les fonctions qui
  s'adressent aux shaders. Les shaders sont des
  scripts situés dans les assets du programme, ils
  permettent de faire des calculs et de modifier
  le rendu d'un calque. Ils sont utilisé ici pour
  les effet d'arrière plan génératifs et pour les
  effets post processing.

**************************************************/ 
                 
                                                
                                                
PShader clippingRVB;                             
PShader clippingHSB;
PShader antialiasing;
PShader round;
PShader blur;
PShader psycho;
PShader smoothEdge;
PShader retro;
PShader grain;
PShader dotted;
PShader motionBlur;
PShader bloom;
PShader fuzz;
PShader rgbOffset;
PShader glitch;
PShader zigzag;
PShader circle;



void shaderInit(){

  /* cette fonction initialise les shaders */

  clippingHSB   = loadShader("shaders/clippingHSB.glsl"  ); 
  clippingRVB   = loadShader("shaders/clippingRVB.glsl"  ); 
  antialiasing  = loadShader("shaders/antialiasing.glsl" );
  round         = loadShader("shaders/round.glsl"        );
  blur          = loadShader("shaders/blur.glsl"         );
  psycho        = loadShader("shaders/psycho.glsl"       );
  smoothEdge    = loadShader("shaders/smoothEdge.glsl"   );
  retro         = loadShader("shaders/retro.glsl"        );
  grain         = loadShader("shaders/grain.glsl"        );
  dotted        = loadShader("shaders/dotted.glsl"       );
  motionBlur    = loadShader("shaders/motionblur.glsl"   );
  bloom         = loadShader("shaders/bloom.glsl"        );
  fuzz          = loadShader("shaders/fuzz.glsl"         );
  rgbOffset     = loadShader("shaders/rgbOffset.glsl"    );
  glitch        = loadShader("shaders/glitch.glsl"       );
  zigzag        = loadShader("shaders/zigzag.glsl"       );
  circle        = loadShader("shaders/circle.glsl"       );
  
  grain.set("opacity",0.06);
  rgbOffset.set("vel",0.5);
  fuzz.set("vel",0.35);
  glitch.set("val",0.2);
  bloom.set("ray",0.004);
  round.set("ray", 0.002);
  
}



void shaderSettings(){

  /* cette fonction donne aux shaders les
  paramètres définis par l'utilisateur*/
  
    psycho.set("amplitude", cntrlFloat[1]);
    psycho.set("divide"   , cntrlFloat[2]);
     retro.set("amplitude", cntrlFloat[1]);
     retro.set("divide"   , cntrlFloat[2]);
    dotted.set("amplitude", cntrlFloat[1]);
    dotted.set("divide"   , cntrlFloat[2]);
    zigzag.set("amplitude", cntrlFloat[1]);
    zigzag.set("divide"   , cntrlFloat[2]);
    circle.set("amplitude", cntrlFloat[1]);
    circle.set("divide"   , cntrlFloat[2]);
}



void shaderDivide( float n ){

  /* cette fonction donne aux shaders les
  paramètres définis par l'utilisateur*/
  
    psycho.set( "divide"   , n );
     retro.set( "divide"   , n );
    dotted.set( "divide"   , n );
    zigzag.set( "divide"   , n );
    circle.set( "divide"   , n );
}



void shaderAmplitude( float n ){

  /* cette fonction donne aux shaders les
  paramètres définis par l'utilisateur*/
  
    psycho.set( "amplitude" , n );
     retro.set( "amplitude" , n );
    dotted.set( "amplitude" , n );
    zigzag.set( "amplitude" , n );
    circle.set( "amplitude" , n );
}



void shaderSetColor( int n ){

  /* cette fonction donne aux shaders les
  couleur de la pallette définie par l'utilisateur*/
  
    psycho.set("palette" , palettes[n]);
     retro.set("palette" , palettes[n]);
    dotted.set("palette" , palettes[n]);
    zigzag.set("palette" , palettes[n]);
    circle.set("palette" , palettes[n]);
}



void shaderBgSet(){

  /* cette fonction donne aux shaders le calque displayLayer
  pour leur permettre de l'utiliser dans leurs effets */

   retro.set("background" , displayLayer);
  dotted.set("background" , displayLayer);
}



void shaderSetInc(){

  /* cette fonction donne aux shaders les valeur relative au temps dans
  le programme pour leur permettre de mettre leurs effets en mouvement*/
  
  psycho.set("inc" , inc );
  zigzag.set("inc" , inc );
  circle.set("inc" , inc );
  
  glitch.set("time" , time);
   grain.set("time" , time);
    fuzz.set("time" , time);
}



void shaderSetLastFrame() {

  /* cette fonction donne aux shaders les dix dernièrs calques preRendering
  pour leur permettre des l'utiliser dans leurs effets */

  motionBlur.set("pastScreen0" , pastScreen[0]);
  motionBlur.set("pastScreen1" , pastScreen[1]);
  motionBlur.set("pastScreen2" , pastScreen[2]);
  motionBlur.set("pastScreen3" , pastScreen[3]);
  motionBlur.set("pastScreen4" , pastScreen[4]);
  motionBlur.set("pastScreen5" , pastScreen[5]);
  motionBlur.set("pastScreen6" , pastScreen[6]);
  motionBlur.set("pastScreen7" , pastScreen[7]);
  motionBlur.set("pastScreen8" , pastScreen[8]);
  motionBlur.set("pastScreen9" , pastScreen[9]);
} 



void setColorNumber(int n){
  
  float r = float(n)/5;
  
  shaderDivide( r );
}  



void setAmplitude(float n){
  
  shaderAmplitude( n );
}  
